@@ -1,4 +1,4 @@
-#       $Id: Rule.pm 1368 2003-08-03 22:06:41Z richardc $
+#       $Id: Rule.pm 1652 2003-10-04 11:19:39Z richardc $
 
 package File::Find::Rule;
 use strict;
@@ -10,7 +10,7 @@ use Carp qw/croak/;
 use File::Find (); # we're only wrapping for now
 use Cwd;           # 5.00503s File::Find goes screwy with max_depth == 0
 
-$VERSION = '0.20_03';
+$VERSION = '0.24_01';
 
 # we'd just inherit from Exporter, but I want the colon
 sub import {
@@ -104,12 +104,14 @@ object if called as class methods.
 sub new {
     my $referent = shift;
     my $class = ref $referent || $referent;
-    bless { rules    => [],  # [0]
-            subs     => [],  # [1]
-            iterator => [],
-            maxdepth => undef,
-            mindepth => undef,
-          }, $class;
+    bless {
+        rules    => [],  # [0]
+        subs     => [],  # [1]
+        iterator => [],
+        extras   => {},
+        maxdepth => undef,
+        mindepth => undef,
+    }, $class;
 }
 
 sub _force_object {
@@ -454,12 +456,21 @@ used.
 Do not apply any tests at levels less than C<$level> (a non-negative
 integer).
 
+=item C<extras( \%extras )>
+
+Specifies extra values to pass through to C<File::File::find> as part
+of the options hash.
+
+For example this allows you to specify following of symlinks like so:
+
+ my $rule = File::Find::Rule->extras({ follow => 1 });
+
 May be invoked many times per rule, but only the most recent value is
 used.
 
 =cut
 
-for my $setter (qw( maxdepth mindepth )) {
+for my $setter (qw( maxdepth mindepth extras )) {
     my $sub = sub {
         my $self = _force_object shift;
         $self->{$setter} = shift;
@@ -469,6 +480,19 @@ for my $setter (qw( maxdepth mindepth )) {
     *$setter = $sub;
 }
 
+
+=item C<relative>
+
+Trim the leading portion of any path found
+
+=cut
+
+sub relative () {
+    my $self = _force_object shift;
+    $self->{relative} = 1;
+    $self;
+}
+
 =item C<not_*>
 
 Negated version of the rule.  An effective shortand related to ! in
@@ -513,23 +537,27 @@ directories.
 sub in {
     my $self = _force_object shift;
 
-    my $cwd = getcwd;
     my @found;
     my $fragment = $self->_compile( $self->{subs} );
     my @subs = @{ $self->{subs} };
 
+    warn "relative mode handed multiple paths - that's a bit silly\n"
+      if $self->{relative} && @_ > 1;
+
+    my $topdir;
     my $code = 'sub {
         (my $path = $File::Find::name)  =~ s#^\./##;
         my @args = ($_, $File::Find::dir, $path);
         my $maxdepth = $self->{maxdepth};
         my $mindepth = $self->{mindepth};
+        my $relative = $self->{relative};
 
         # figure out the relative path and depth
         my $relpath = $File::Find::name;
-        $relpath =~ s{^\Q$File::Find::topdir\E/?}{};
+        $relpath =~ s{^\Q$topdir\E/?}{};
         my $depth = scalar File::Spec->splitdir($relpath);
         #print "name: \'$File::Find::name\' ";
-        #print "relpath: \'$relpath\' depth: $depth\n";
+        #print "relpath: \'$relpath\' depth: $depth relative: $relative\n";
 
         defined $maxdepth && $depth >= $maxdepth
            and $File::Find::prune = 1;
@@ -542,7 +570,12 @@ sub in {
         my $discarded;
         return unless ' . $fragment . ';
         return if $discarded;
-        push @found, $path;
+        if ($relative) {
+            push @found, $relpath if $relpath ne "";
+        }
+        else {
+            push @found, $path;
+        }
     }';
 
     #use Data::Dumper;
@@ -550,7 +583,12 @@ sub in {
     #print "Compiled sub: '$code'\n";
 
     my $sub = eval "$code" or die "compile error '$code' $@";
-    File::Find::find( $sub, @_ );
+    my $cwd = getcwd;
+    for my $path (@_) {
+        # $topdir is used for relative and maxdepth
+        $topdir = $path;
+        File::Find::find( { %{ $self->{extras} }, wanted => $sub }, $path );
+    }
     chdir $cwd;
 
     return @found;
